<TeXmacs|1.0.6.10>

<style|amsart>

<\body>
  <action|EXPAND|(quail "expand()")>

  <doc-data|<doc-title|alias.tm>>

  <\table-of-contents|toc>
    <vspace*|1fn><with|font-series|bold|math-font-series|bold|Table of
    contents> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
    <no-break><pageref|auto-1><vspace|0.5fn>
  </table-of-contents>

  <\code>
    @@ buttons

    <with|color|dark red|def> <with|color|blue|expand>():

    \ \ <with|color|dark green|pass>
  </code>

  <section|A declaration dispatcher>

  The declaration dispatcher receives alias declarations in the quail modules
  and notifies observers that the declaration occurs.

  <\code>
    @@ py alias

    <with|color|dark green|from> inspect <with|color|dark green|import>
    currentframe

    <with|color|dark green|import> datetime

    \;

    <with|color|dark red|class> <with|color|#800080|aDeclarationObserver>(object):

    \ \ <with|font-family|rm|An observer can register to receive alias
    declarations from modules defined in>

    \ \ <with|font-family|rm|the python sections of texmacs pages. To define
    an observer, just subclass this>

    \ \ <with|font-family|rm|base class and redefine the method
    <verbatim|doHandleDeclaration> which is called for>

    \ \ <with|font-family|rm|each declaration when the observer is
    registered.>

    \ \ <with|color|dark red|def> <with|color|blue|doHandleDeclaration>(o,

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ theObjectPath, theAlias, thePageName,
    theSection):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|def> <with|color|blue|doRegister>(o):

    \ \ \ \ TheDeclarationDispatcher doAddObserver(o)

    \ \ <with|color|dark red|def> <with|color|blue|doUnregister>(o):

    \ \ \ \ TheDeclarationDispatcher doRemoveObserver(o)

    \;

    <with|color|dark red|class> <with|color|#800080|aDeclarationDispatcher>(object):

    \ \ <with|font-family|rm|The declaration dispatcher manages observers
    registrations and notifies the observers>

    \ \ <with|font-family|rm|that a declaration occured. This is a singleton
    class.>

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o):

    \ \ \ \ o _itsObservers = set()

    \ \ <with|color|dark red|def> <with|color|blue|doAddObserver>(o,
    theObserver):

    \ \ \ \ o _itsObservers add(theObserver)

    \ \ <with|color|dark red|def> <with|color|blue|doRemoveObserver>(o,
    theObserver):

    \ \ \ \ o _itsObservers discard(theObserver)

    \ \ @property

    \ \ <with|color|dark red|def> <with|color|blue|itsObservers>(o):

    \ \ \ \ <with|color|dark green|return> list(o _itsObservers)

    \ \ <with|color|dark red|def> <with|color|blue|doHandleDeclaration>(o,

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ theObjectPath, theAlias, thePageName,
    theSection):

    \ \ \ \ <with|color|dark green|for> theObserver <with|color|dark
    green|in> o itsObservers:

    \ \ \ \ \ \ theObserver doHandleDeclaration(

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ theObjectPath, theAlias, thePageName,
    theSection)

    \;

    \ <with|font-family|rm|<math|\<searrow\>> The singleton instance of
    <verbatim|aDeclarationDispatcher.>>

    TheDeclarationDispatcher = aDeclarationDispatcher()

    \;

    <with|color|dark red|def> <with|color|blue|doDeclareAlias>(theObjectPath,
    theAlias, theGlobals=None):

    \ \ <with|font-family|rm|<math|\<nearrow\>> This function is the one used
    by modules to declare quail aliases.>

    \ \ <with|color|dark green|assert> isinstance(theObjectPath, str)

    \ \ <with|color|dark green|if> theGlobals <with|color|dark green|is>
    None:

    \ \ \ \ theFrame = currentframe()

    \ \ \ \ <with|color|dark green|try>:

    \ \ \ \ \ \ theGlobals = theFrame f_back f_globals

    \ \ \ \ <with|color|dark green|finally>:

    \ \ \ \ \ \ <with|color|dark green|del> theFrame

    \ \ theFile = theGlobals["__file__"]

    \ \ theFolder, theName = theFile splitpath()

    \ \ thePageName = theFolder + ".tm"

    \ \ <with|color|dark green|assert>(thePageName isfile())

    \ \ <with|color|dark green|assert>(theFile ext <with|color|dark green|in>
    (".py", ".pyc", ".pyo"))

    \ \ theSection = theName[:-len(theFile ext)]

    \ \ TheDeclarationDispatcher doHandleDeclaration(

    \ \ \ \ \ \ \ \ \ theObjectPath, theAlias, thePageName, theSection)
  </code>

  <section|The alias database creator>

  The database will be stored as a file <verbatim|resource/aliasDB.py>. The
  following program creates the database by examining the alias declarations
  contained in quail's texmacs sources.

  <\code>
    @@ py databaseCreator

    \;

    <with|color|dark green|import> datetime

    <with|color|dark green|from> pprint <with|color|dark green|import>
    pformat

    <with|color|dark green|from> quailCenter <with|color|dark green|import>
    quailGet

    root = quailGet("the quail directory")

    page = quailGet("the page class")

    path = quailGet("the path class")

    alias = page(root/"alias.tm") alias

    aDeclarationObserver = alias aDeclarationObserver

    \;

    <with|color|dark red|class> <with|color|#800080|anIndexDict>(dict):

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o, *args,
    **kwd):

    \ \ \ \ dict __init__(o, *args, **kwd)

    \ \ \ \ o itsNextIndex = 0

    \ \ <with|color|dark red|def> <with|color|blue|__missing__>(o, key):

    \ \ \ \ o[key] = o itsNextIndex

    \ \ \ \ o itsNextIndex += 1

    \ \ \ \ <with|color|dark green|return> o[key]

    \;

    <with|color|dark red|class> <with|color|#800080|aDatabaseCreator>(aDeclarationObserver):

    \ \ <with|font-family|rm|<math|\<nearrow\>> This class is used to create
    the alias database from fresh.>

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o):

    \ \ \ \ aDeclarationObserver __init__(o)

    \ \ \ \ o itsPageDict = anIndexDict() <with|font-family|rm|A dictionary
    <em|path to page> <math|\<rightarrow\>> <em|id>.>

    \ \ \ \ o itsSectionDict = anIndexDict()

    \ \ \ \ o itsAliasDict = anIndexDict()

    \ \ <with|color|dark red|def> <with|color|blue|doCreateDatabase>(o):

    \ \ \ \ o doRegister()

    \ \ \ \ o doLoadTmModules(root)

    \ \ \ \ o doUnregister()

    \ \ \ \ o doDumpDatabase(root/"resource"/"aliasDB.py")

    \ \ <with|color|dark red|def> <with|color|blue|doHandleDeclaration>(o,

    \ \ \ \ \ \ \ \ \ \ \ theObjectPath, theAlias, thePageName, theSection):

    \ \ \ \ <with|font-family|rm|<math|\<nearrow\>> This declaration handler
    fills the object dicts according to the declaration.>

    \ \ \ \ thePageName = thePageName[len(root)+1:]

    \ \ \ \ thePageId = o itsPageDict[thePageName]

    \ \ \ \ thePair = (thePageId, theSection)

    \ \ \ \ theSectionId = o itsSectionDict[thePair]

    \ \ \ \ p = tuple(word.strip() <with|color|dark green|for> word
    <with|color|dark green|in> theObjectPath.split("."))

    \ \ \ \ p = () <with|color|dark green|if> p == ("",) <with|color|dark
    green|else> p

    \ \ \ \ theTuple = (theSectionId, p, theAlias.strip())

    \ \ \ \ theAliasId = o itsAliasDict[theTuple]

    \ \ <with|color|dark red|def> <with|color|blue|doDumpDatabase>(o,
    thePath):

    \ \ \ \ theFileOut = open(thePath, "w")

    \ \ \ \ theTemplate = """

    import datetime

    itsCreationDate = %(theCreationTime)s

    itsPageDict = \\\\\\n%(thePages)s

    itsSectionDict = \\\\\\n%(theSections)s

    itsAliasDict = \\\\\\n%(theAliases)s

    \ \ \ \ """

    \ \ \ \ thePageDict = dict((ixp, p)

    \ \ \ \ \ \ \ <with|color|dark green|for> (p, ixp) <with|color|dark
    green|in> o itsPageDict.iteritems())

    \ \ \ \ theSectionDict = dict( (ixs, (ixp, s))

    \ \ \ \ \ \ \ <with|color|dark green|for> ((ixp, s), ixs)
    <with|color|dark green|in> o itsSectionDict.iteritems())

    \ \ \ \ theAliasDict = dict( (a, (ixa, ixs, p))

    \ \ \ \ \ \ \ <with|color|dark green|for> ((ixs, p, a), ixa)
    <with|color|dark green|in> o itsAliasDict.iteritems())

    \ \ \ \ theFileOut.write(theTemplate % {

    \ \ \ \ \ \ "theCreationTime" : repr(datetime.datetime.now()),

    \ \ \ \ \ \ "thePages": pformat(thePageDict),

    \ \ \ \ \ \ "theSections": pformat(theSectionDict),

    \ \ \ \ \ \ "theAliases": pformat(theAliasDict),

    \ \ \ \ })

    \ \ \ \ theFileOut close()

    \;

    \ \ <with|color|dark red|def> <with|color|blue|doLoadTmModules>(o,
    theDirectory):

    \ \ \ \ <with|font-family|rm|Finds all texmacs pages from
    <verbatim|theDirectory>, parses them and load the modules.>

    \ \ \ \ <with|color|dark green|for> thePageName <with|color|dark
    green|in> theDirectory walkfiles("*.tm"):

    \ \ \ \ \ \ thePage = page(thePageName)

    \ \ \ \ \ \ #print(thePageName)

    \ \ \ \ \ \ theTagSet = set()

    \ \ \ \ \ \ <with|color|dark green|for> s <with|color|dark green|in>
    thePage sesSections:

    \ \ \ \ \ \ \ \ theTag = s sonTag

    \ \ \ \ \ \ \ \ <with|color|dark green|if> theTag <with|color|dark
    green|in> theTagSet:

    \ \ \ \ \ \ \ \ \ \ <with|color|dark green|continue>

    \ \ \ \ \ \ \ \ theTagSet add(theTag)

    \ \ \ \ \ \ \ \ #print theTag

    \ \ \ \ \ \ \ \ <with|color|dark green|if> theTag[0] != "py":

    \ \ \ \ \ \ \ \ \ \ <with|color|dark green|continue>

    \ \ \ \ \ \ \ \ theModuleName = "_" join(theTag[1:])

    \ \ \ \ \ \ \ \ <with|color|dark green|try>:

    \ \ \ \ \ \ \ \ \ \ #print("trying %s" % theModuleName)

    \ \ \ \ \ \ \ \ \ \ getattr(thePage, theModuleName)
    <with|font-family|rm|Loads the module.>

    \ \ \ \ \ \ \ \ \ \ #print("success")

    \ \ \ \ \ \ \ \ <with|color|dark green|except> Exception:

    \ \ \ \ \ \ \ \ \ \ #print("failed")

    \ \ \ \ \ \ \ \ \ \ <with|color|dark green|raise>

    \;

    <with|color|dark green|if> __name__ == "__main__":

    \ \ theCreator = aDatabaseCreator()

    \ \ theCreator doCreateDatabase()

    \;
  </code>

  \;

  <action|EXPAND|(quail "expand()")>
</body>

<\initial>
  <\collection>
    <associate|bg-color|#e0e0e0>
    <associate|language|english>
    <associate|magnification|1.1>
    <associate|page-type|# Simply write the paper name. See papersize(5) for
    possible values.>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|?|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|2|?>>
    <associate|auto-4|<tuple|3|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Table
      of contents> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1.<space|2spc>The
      alias database> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|2.<space|2spc>some
      algorithms> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|3.<space|2spc>A
      declaration dispatcher> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>