<TeXmacs|1.0.6.10>

<style|amsart>

<\body>
  <action|EXPAND|(quail "expand()")>

  <doc-data|<doc-title|parseScm.tm>>

  <\table-of-contents|toc>
    <vspace*|1fn><with|font-series|bold|math-font-series|bold|Table of
    contents> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
    <no-break><pageref|auto-1><vspace|0.5fn>
  </table-of-contents>

  <\code>
    @@ buttons

    <with|color|dark red|def> <with|color|blue|expand>():

    \ \ <with|color|dark green|pass>
  </code>

  <section|Parsing texmacs scheme documents>

  TeXmacs possesses the ability to convert texmacs file to a scheme format.
  This scheme format is very easy to parse for python because the scheme
  document looks like a python tuple containing labels, literal strings and
  other tuples, with the difference that there is no separating commas
  between the tuple elements. The file can be parsed by looking only for the
  characters <verbatim|">, <verbatim|(> and <verbatim|)> and escaped
  characters, and transformed in a single python tuple containing other
  tuples. There are 2 parsing contexts (inside a character string and outside
  of a character string).

  <section|A texmacs scheme parser>

  <subsection|The parser>

  <\code>
    @@ py schemeParser

    <with|color|dark green|from> quailCenter <with|color|dark green|import>
    doDeclareAlias, quailGet

    doDeclareAlias("", "the scheme parser module")

    <with|color|dark green|from> collections <with|color|dark green|import>
    deque

    \;

    <with|color|dark red|class> <with|color|#800080|aSchemeParser>(object):

    \ \ <with|color|dark red|proc> <with|color|blue|__init__>(o):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|proc> <with|color|blue|parse>(o, theContent):

    \ \ \ \ o itsContent = theContent

    \ \ \ \ o itsStack = deque([[]])

    \ \ \ \ contexts = quailGet("the scheme parsing context module")

    \ \ \ \ o itsScmCtxString = contexts aScmCtxString(o)

    \ \ \ \ o itsScmCtxNormal = contexts aScmCtxNormal(o)

    \ \ \ \ o itsContext = o itsScmCtxNormal

    \ \ \ \ o itsPosition = 0

    \ \ \ \ events = quailGet("the scheme parsing events module")

    \ \ \ \ <with|color|dark green|for> event, pos <with|color|dark green|in>
    events.generateSchemeEvents(o itsContent):

    \ \ \ \ \ \ getattr(o itsContext, event)(pos)

    \ \ \ \ <with|color|dark green|assert>(len(o itsStack) == 1)

    \ \ \ \ <with|color|dark green|assert>(len(o itsStack[0]) == 1)

    \ \ \ \ <with|color|dark green|return> o itsStack[0][0]

    \ \ <with|color|dark red|proc> <with|color|blue|getText>(o, pos,
    nextPos):

    \ \ \ \ s = o itsContent[o itsPosition: pos]

    \ \ \ \ o itsPosition = nextPos

    \ \ \ \ <with|color|dark green|return> s

    \ \ <with|color|dark red|class> <with|color|#800080|aScmItem>(object):

    \ \ \ \ <with|font-family|rm|The parser produces a tree, which is just a
    tuple containing strings, other>

    \ \ \ \ <with|font-family|rm|similar tuples, or <em|scheme items>. This
    is the python class for these items.>

    \ \ \ \ <with|color|dark red|def> <with|color|blue|__init__>(o,
    theString):

    \ \ \ \ \ \ o itsContent = theString

    \ \ @staticmethod

    \ \ <with|color|dark red|def> <with|color|blue|isScmOperatorTree>(theTuple):

    \ \ \ \ <with|font-family|rm|Checks that the argument has the structure
    of a scheme tree organized like this:>

    \ \ \ \ <with|font-family|rm|The tuple is not empty, and it's first
    element is an operator (a scheme item), the>

    \ \ \ \ <with|font-family|rm|other elements are strings or similar
    tuple.>

    \ \ \ \ <with|color|dark green|if> ((<with|color|dark green|not>
    isinstance(theTuple, tuple)) <with|color|dark green|or>

    \ \ \ \ \ \ (len(theTuple) == 0) <with|color|dark green|or>
    (<with|color|dark green|not> isinstance(theTuple[0], aScmItem))):

    \ \ \ \ \ \ <with|color|dark green|return> False

    \ \ \ \ <with|color|dark green|for> theElement <with|color|dark green|in>
    theTuple[1:]:

    \ \ \ \ \ \ <with|color|dark green|if> isinstance(theElement, str):

    \ \ \ \ \ \ \ \ <with|color|dark green|continue>

    \ \ \ \ \ \ <with|color|dark green|if> (<with|color|dark green|not>
    aSchemeParser isScmOperatorTree(theElement)):

    \ \ \ \ \ \ \ \ <with|color|dark green|return> False

    \ \ \ \ <with|color|dark green|return> True

    doDeclareAlias("aSchemeParser", "the scheme parser class")

    aScmItem = aSchemeParser aScmItem
  </code>

  <subsection|The parsing contexts>

  <\code>
    @@ py schemeContext

    <with|color|dark green|from> quailCenter <with|color|dark green|import>
    doDeclareAlias

    doDeclareAlias("", "the scheme parsing context module")

    import re

    theEscapedPattern = re compile(r"\\\\.", re.DOTALL)

    def subEscaped(match):

    \ \ return match.group(0)[1]

    \ \ 

    \;

    <with|color|dark red|class> <with|color|#800080|aScmCtxBase>(object):

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o):

    \ \ \ \ <with|color|dark green|pass>

    \;

    <with|color|dark red|class> <with|color|#800080|aScmCtxString>(aScmCtxBase):

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o, theParser):

    \ \ \ \ aScmCtxBase __init__(o)

    \ \ \ \ o itsParser = theParser

    \ \ <with|color|dark red|def> <with|color|blue|evScmEsc>(o, pos):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|def> <with|color|blue|evScmB>(o, pos):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|def> <with|color|blue|evScmE>(o, pos):

    \ \ \ \ <with|color|dark green|pass>

    \ \ <with|color|dark red|def> <with|color|blue|evScmQ>(o, pos):

    \ \ \ \ s = o itsParser getText(pos+1, pos+1)

    \ \ \ \ s = eval(s)

    \ \ \ \ s = theEscapedPattern.sub(subEscaped, s)

    \ \ \ \ o itsParser itsStack[-1] append(s)

    \ \ \ \ o itsParser itsContext = o itsParser itsScmCtxNormal

    \;

    <with|color|dark red|class> <with|color|#800080|aScmCtxNormal>(aScmCtxBase):

    \ \ <with|color|dark red|def> <with|color|blue|__init__>(o, theParser):

    \ \ \ \ aScmCtxBase __init__(o)

    \ \ \ \ o itsParser = theParser

    \ \ <with|color|dark red|def> <with|color|blue|evScmQ>(o, pos):

    \ \ \ \ o collectCode(pos, pos)

    \ \ \ \ o itsParser itsContext = o itsParser itsScmCtxString

    \ \ <with|color|dark red|def> <with|color|blue|evScmB>(o, pos):

    \ \ \ \ o collectCode(pos, pos+1)

    \ \ \ \ L = list()

    \ \ \ \ o itsParser itsStack append(L)

    \ \ <with|color|dark red|def> <with|color|blue|evScmE>(o, pos):

    \ \ \ \ o collectCode(pos, pos+1)

    \ \ \ \ L = o itsParser itsStack pop()

    \ \ \ \ o itsParser itsStack[-1] append(tuple(L))

    \ \ <with|color|dark red|def> <with|color|blue|collectCode>(o, pos,
    nextPos):

    \ \ \ \ s = o itsParser getText(pos, nextPos)

    \ \ \ \ s = s.strip()

    \ \ \ \ <with|color|dark green|if> s:

    \ \ \ \ \ \ o itsParser itsStack[-1] append(o itsParser aScmItem(s))

    \;
  </code>

  <subsection|Generating parse events>

  <\code>
    @@ py schemeEvents

    <with|color|dark green|from> quailCenter <with|color|dark green|import>
    doDeclareAlias

    doDeclareAlias("", "the scheme parsing events module")

    <with|color|dark green|import> re

    \;

    _theSchemeEventMap = {

    \ \ "\\\\" : "evScmEsc",

    \ \ "(" \ : "evScmB",

    \ \ ")" \ : "evScmE",

    \ \ '"' \ : "evScmQ",

    }

    \;

    _theSchemePattern = re.compile(r"""

    \ \ ["()] \ \ \ <with|font-family|rm|<math|\<leftarrow\>> a double quote
    or a parenthesis.>

    \ \ \| \\\\. \ \ \ <with|font-family|rm|<math|\<leftarrow\>> or an
    escaped character>

    """, re.VERBOSE\|re DOTALL)

    \;

    <with|color|dark red|def> <with|color|blue|generateSchemeEvents>(theContent):

    \ \ <with|color|dark green|for> match <with|color|dark green|in>
    _theSchemePattern finditer(theContent):

    \ \ \ \ s = match.group(0)

    \ \ \ \ <with|color|dark green|yield> (_theSchemeEventMap[s[0]],
    match.start())
  </code>

  <section|Converting scheme tuple trees>

  Tuple trees can be converted into texmacs trees similar to trees produced
  by the texmacs parser. The following module is devoted to this task

  <\code>
    @@ py converter

    <with|color|dark green|from> quailCenter <with|color|dark green|import>
    doDeclareAlias, quailGet

    doDeclareAlias("", "the scheme parser converter module")

    \;

    <with|color|dark red|def> <with|color|blue|getTagSet>(theTupleTree):

    \ \ <with|font-family|rm|Returns the set of all the texmacs tags in the
    tuple tree (assuming it's a scheme>

    \ \ <with|font-family|rm|operator tree).>

    \ \ theSet = set()

    \ \ doFillTagSet(theTupleTree, theSet)

    \ \ <with|color|dark green|return> theSet

    \;

    <with|color|dark red|def> <with|color|blue|doFillTagSet>(theTupleTree,
    theSet):

    \ \ <with|font-family|rm|Helper recursive function for
    <verbatim|getTagSet>.>

    \ \ theSet add(theTupleTree[0] itsContent)

    \ \ <with|color|dark green|for> theElement <with|color|dark green|in> (

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ x <with|color|dark green|for> x
    <with|color|dark green|in> theTupleTree[1:] <with|color|dark green|if>
    isinstance(x, tuple)):

    \ \ \ \ doFillTagSet(theElement, theSet)

    \;

    <with|color|dark red|def> <with|color|blue|doConvertToTmTree>(theTupleTree):

    \ \ TheTmTreeClasses = quailGet(

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ "the texmacs tree module")
    TheTmTreeClasses

    \ \ tmTagToWord = quailGet("texmacs tag to word")

    \ \ theName = tmTagToWord(theTupleTree[0] itsContent)

    \ \ L = list(theTupleTree[1:])

    \ \ <with|color|dark green|for> i, x <with|color|dark green|in>
    enumerate(L):

    \ \ \ \ <with|color|dark green|if> isinstance(x, tuple):

    \ \ \ \ \ \ L[i] = doConvertToTmTree(x)

    \ \ if ((theName == "tm_style") and (len(L) == 1)

    \ \ \ \ \ \ \ \ \ \ \ and (L[0] __class__ __name__ == "tm_tuple")):

    \ \ \ \ <with|font-family|rm|Texmacs translates
    <verbatim|\<less\>style\|amsart\<gtr\>> to <verbatim|(style (tuple
    "amsart"))> in scheme.>

    \ \ \ \ <with|font-family|rm|This hack removes the <verbatim|tuple> to
    conform to the texmacs parser.>

    \ \ \ \ L = list(L[0])

    \ \ theTree = TheTmTreeClasses[theName](L)

    \ \ <with|color|dark green|return> theTree

    \;
  </code>

  \;

  <action|EXPAND|(quail "expand()")>
</body>

<\initial>
  <\collection>
    <associate|bg-color|#e0e0e0>
    <associate|language|english>
    <associate|magnification|1.1>
    <associate|page-type|# Simply write the paper name. See papersize(5) for
    possible values.>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|?|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|2|?>>
    <associate|auto-4|<tuple|2.1|?>>
    <associate|auto-5|<tuple|2.2|?>>
    <associate|auto-6|<tuple|2.3|?>>
    <associate|auto-7|<tuple|3|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Table
      of contents> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1.<space|2spc>Parsing
      texmacs scheme documents> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|2.<space|2spc>A
      texmacs scheme parser> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|2.1.<space|2spc>The parser
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|2.2.<space|2spc>The parsing contexts
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|2.3.<space|2spc>Generating parse events
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|3.<space|2spc>Converting
      scheme tuple trees> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>